import wikipedia as wp
import wptools
from pysymbolscanner.infobox import Infobox
from pysymbolscanner.const import (
    blocklist_search,
    most_common_endings,
    remove_most_common_endings,
)
import requests
from bs4 import BeautifulSoup


def _get_infobox_of_page(name, check_item, lang):
    try:
        if lang == 'es':
            opt = {
                'boxterm': 'Ficha',
                'skip': ['imageinfo'],
                'silent': True,
                'lang': lang,
            }
        else:
            opt = {'skip': ['imageinfo'], 'silent': True, 'lang': lang}
        page = wptools.page(name, **opt).get_parse()
        # search item must be in data
        search_str = str(page.data.get('wikitext', '')).lower()
        check_item_search = remove_most_common_endings(check_item).lower()
        name_search = remove_most_common_endings(name).lower()
        ctx_name = max(
            (
                search_str.count(check_item_search),
                search_str.count(name_search),
            )
        )
        if name_search != check_item_search and ctx_name < 5:
            return None
        infobox = page.data['infobox']
        if infobox:
            infobox = {k.lower(): v for k, v in infobox.items()}
        return infobox
    except LookupError:
        return None


def _is_infobox(infobox):
    if infobox is None:
        return False
    infobox_items = [
        'nam',
        'effectif',
        'date de création',
        'siège (pays)',
        'name',
        'foundation',
        'hq_location_country',
        'unternehmen',
        'gründung_verein',
        'location',
        'industry',
        'num_employees',
        'traded_as',
        'isin',
        'gründungsdatum',
        'mitarbeiterzahl',
        'nombre',
        'empleados',
        'sede',
        'sitz',
    ]
    ctx = sum(map(lambda x: 1 if x in infobox else 0, infobox_items))
    if ctx > 1:
        return True
    return False


def _is_in_infobox(infobox, search):
    search = search.replace('Rosagro', 'Rusagro')
    search_items = [search] if len(search.split()) == 0 else search.split()
    values = list(filter(lambda x: x not in most_common_endings, search_items))
    ctx = 0
    for value in values:
        if any(
            map(
                lambda x, val=value: val.lower() in x.lower(), infobox.values()
            )
        ):
            ctx += 1
    result = ctx / len(values) > 0.5
    return result


def get_wiki_infobox(page_search, lang_codes=['en', 'de', 'es', 'fr']):
    for lang in lang_codes:
        wp.set_lang(lang)
        search = filter(
            lambda x: x not in blocklist_search,
            wp.search(page_search, results=3),
        )
        if not search:
            continue
        for item in search:
            infobox = _get_infobox_of_page(item, page_search, lang)
            if _is_infobox(infobox):
                return infobox, lang
    return None


def get_infobox(page_search, lang_codes=['en', 'de', 'es', 'fr']):
    infobox = get_wiki_infobox(page_search, lang_codes)
    if infobox is None or infobox[1] is None:
        return None
    parsed_infobox = Infobox.from_wiki_infobox(*infobox)
    if not parsed_infobox.name:
        parsed_infobox.name = page_search
        parsed_infobox.names.append(page_search)
    return parsed_infobox


def get_wiki_page_title_and_links(link, lang_codes):
    get_url = requests.get(link)
    get_text = get_url.text
    soup = BeautifulSoup(get_text, "html.parser")
    company = soup.find('h1').text
    links = soup.findAll(
        'a', href=True, attrs={'class': 'interlanguage-link-target'}
    )
    links = list(
        filter(
            lambda x: x[1] in lang_codes,
            map(lambda x: (x['href'], x['lang']), links),
        )
    )
    return company, links


def get_merged_infobox(page_search, link, link_lang, lang_codes=None):
    if lang_codes is None:
        lang_codes = ['en', 'de', 'es', 'fr']
    result = None
    page_search_dict = dict(map(lambda x: (x, None), lang_codes))

    # find all wikipages for company name
    if link and link_lang:
        wiki_url = get_wiki_url(link_lang, link.replace('/wiki/', ''))
        company, links = get_wiki_page_title_and_links(
            wiki_url, lang_codes
        )
        page_search_dict[link_lang] = company
        for wiki_link, wiki_link_lang in links:
            company, _ = get_wiki_page_title_and_links(
                wiki_link, lang_codes
            )
            page_search_dict[wiki_link_lang] = company

    # get all infobox data for each wikipage
    for lang in lang_codes:
        if not page_search_dict[lang]:
            continue
        infobox = get_infobox(page_search_dict[lang], [lang])
        if infobox is None:
            continue

        if result:
            result.update(infobox)
        else:
            result = infobox

    return result


def get_wiki_url(lang, title):
    return f'https://{lang}.wikipedia.org/wiki/{title}'



import re
import pycountry
import gettext
from unidecode import unidecode
from pysymbolscanner.stock import Stock
from pysymbolscanner.word_score import get_best_match


class Infobox:
    def __init__(
        self, name, founded, employees, loc, industry, symbols, isins
    ):
        self.name = name
        self.names = [name] if name else []
        self.founded = founded
        self.employees = employees
        self.location = loc
        self.industry = industry
        self.symbols = symbols
        self.isins = isins

    @classmethod
    def from_wiki_infobox(cls, infobox, lang):
        if not infobox:
            return None
        infobox_items = parse_infobox(infobox, lang)
        if not infobox_items:
            return None
        return cls(*infobox_items)

    def to_stock(self, index, symbol):
        symbols = [symbol] + self.symbols
        return Stock(
            '',
            self.name,
            '',
            '',
            '' if not symbols else symbols[0],
            self.location,
            index,
            self.industry,
            self.symbols,
            self.isins,
            self.founded,
            self.employees,
        )

    def update(self, infobox):
        if not self.name and infobox.name:
            self.name = long_to_short(infobox.name)
            self.names.append(self.name)
        elif infobox.name:
            self.names.append(long_to_short(infobox.name))
            # we take the name with the most occurrences
            if (
                len(set(map(self.names.count, self.names))) > 1
            ):  # checks if there is a max value
                self.name = long_to_short(
                    max(set(self.names), key=self.names.count)
                )
            else:
                self.name = self.names[0]
        if not self.founded:
            self.founded = infobox.founded
        if not self.employees:
            self.employees = infobox.employees
        if not self.location:
            self.location = infobox.location
        if not self.industry:
            self.industry = infobox.industry
        if not self.symbols:
            self.symbols = infobox.symbols
        elif infobox.symbols:
            self.symbols = list(set(self.symbols + infobox.symbols))
        elif infobox.isins:
            self.isins = list(set(self.isins + infobox.isins))


def long_to_short(name):
    result = name.replace('Aktiengeselschaft', 'AG')
    result = result.replace(', Inc.', ' Inc.')
    result = re.sub(r' \(.+\)$', '', result)
    if not result or not result.strip():
        raise RuntimeError(f'Error during name processing {name}')
    return result


def clean_html(raw_html):
    cleaner = re.compile('<.*?>')
    cleantext = re.sub(cleaner, ' ', raw_html)
    cleantext = re.sub(' +', ' ', cleantext)
    return cleantext.strip()


def get_value(infobox, keys):
    return next(
        filter(
            lambda x: x is not None,
            map(lambda key: infobox.get(key, None), keys),
        ),
        '',
    )


def get_location(
    infobox,
    lang,
    keys=[
        'native_name_lang',
        'location_country',
        'hq_location_country',
        'hq_location',
        'location',
        'land',
        'sitz',
        'siège (pays)',
        'sede',
    ],
):
    loc = get_value(infobox, keys)
    loc_translated = get_country(lang, loc if loc else str(infobox))
    if loc_translated:
        loc = loc_translated[0]
    return loc


def get_foundation_date(
    infobox,
    keys=[
        'foundation',
        'founded',
        'gründungsdatum',
        'date de création',
        'dates-clés',
        'fundación',
        'gründung_verein',
    ],
):
    founded = get_value(infobox, keys)
    if founded:
        founded = re.findall(r'\d{4}', founded)
        if founded:
            return int(min(founded))
    return ''


def get_employees(
    infobox, keys=['num_employees', 'mitarbeiterzahl', 'effectif', 'empleados']
):
    employees = get_value(infobox, keys)
    employees = employees.replace(',', '').replace('.', '')
    employees_items = re.findall(r'\d+', employees)
    if employees_items:
        employees = int(employees_items[0])
    return employees


def get_name(infobox, keys=['name', 'nam', 'nombre', 'unternehmen']):
    name = get_value(infobox, keys)
    regex_values = [
        r'{{color\|.+?\|(.+?)}}',
        r'{{[a-z]+\|(.+?)}}',
        r'\[\[#(.+?)\|.+?\]\]',
    ]
    for regex in regex_values:
        names = re.findall(regex, name)
        if names:
            return names[0]

    return clean_html(name)


def get_isins(infobox, keys=['isin', 'isin2', 'isin3', 'isin4']):
    result = []
    for key in keys:
        val = get_value(infobox, [key])
        if not val:
            continue
        isins = re.findall(r'([A-Z]{2}[A-Z0-9]{9}[0-9]{1})', val)
        if isins:
            result += isins
    return result


def get_symbols(infobox, keys=['traded_as', 'action', 'símbolo_bursátil']):
    val = get_value(infobox, keys)
    val = val.replace(r'\_', '')
    symbols = re.findall(r'{{([a-zA-Z0-9_: ]+)}}', val)
    if not symbols:
        symbols = re.findall(r'{{([a-zA-Z0-9_|\- ]+)}}', val)
    return symbols


def find_by_alpha_code(wiki_str, translate):
    result = None
    # find by alpha 2 or alpha 3 code
    for word_length in [3, 2]:
        if result:
            break

        extract = list(
            filter(
                lambda x, length=word_length: len(x) == length,
                wiki_str.lower().replace('.', '').split(' '),
            )
        )

        if not extract:
            continue

        word = extract[-1]
        result = next(
            filter(
                lambda key, length=word_length, alpha=word: translate[key][
                    length - 1
                ].lower()
                == alpha,
                translate,
            ),
            None,
        )
    return result


def get_country(loc, mystr):
    if not mystr:
        return None
    mystr = mystr.replace('.', '')
    mystr = clean_html(mystr)
    mystr = re.sub(r'[^0-9a-zA-Z _-]+', ' ', unidecode(mystr)).strip()
    mystr = mystr.replace('Russia', 'Russian Federation')
    mystr = mystr.replace('Russland', 'Russische Föderation')
    mystr = mystr.replace('England', 'United Kingdom')
    mystr = 'United States' if 'California' in mystr else mystr
    mystr = 'United States' if 'Florida' in mystr else mystr
    mystr = 'United Kingdom' if 'Jersey' in mystr else mystr
    if loc != 'en':
        # load language of wiki page
        lang = gettext.translation(
            'iso3166', pycountry.LOCALES_DIR, languages=[loc]
        )
        lang.install()
        _ = lang.gettext
        translate = dict(
            map(
                lambda x: (_(x.name), (x.name, x.alpha_2, x.alpha_3)),
                pycountry.countries,
            )
        )
    else:
        translate = dict(
            map(
                lambda x: (x.name, (x.name, x.alpha_2, x.alpha_3)),
                pycountry.countries,
            )
        )

    # try to find the occurrence of country names in sentences
    country = next(
        filter(
            lambda key: unidecode(key.lower()) in unidecode(mystr.lower()),
            translate,
        ),
        None,
    )

    # try to find by alpha code
    if not country:
        country = find_by_alpha_code(mystr, translate)

    # try to find by similarity
    if not country:
        vals = list(translate.keys())
        words = mystr.split() if mystr.split() else mystr
        idx, score = max(
            map(lambda x: get_best_match(x, vals), words), key=lambda x: x[1]
        )
        if score > 0.8:
            country = vals[idx]
    # translate back to english
    if country:
        country = translate[country]
    return country


def parse_infobox(infobox, lang):
    if infobox is None:
        return None
    name = get_name(infobox)
    loc = get_location(infobox, lang)
    founded = get_foundation_date(infobox)
    employees = get_employees(infobox)

    industry = []
    if lang == 'en':
        industry = re.findall(
            r'\[[a-zA-Z_ ()]+\|([a-zA-Z_ ]+)\]', infobox.get('industry', '')
        )
        if not industry:
            industry = re.findall(
                r'\[\[([a-zA-Z_ ]+)\]\]', infobox.get('industry', '')
            )
    symbols = get_symbols(infobox)
    isins = get_isins(infobox)
    return name, founded, employees, loc, industry, symbols, isins



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pysymbolscanner
  Copyright 2020 Slash Gordon
  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""

import logging
import os
import pickle
import sys
import pandas as pd
import multiprocessing
from pysymbolscanner.index_definitions import Indices
from pysymbolscanner.wiki import get_merged_infobox, get_wiki_url
from pysymbolscanner.stock import Stock
from pysymbolscanner.word_score import deep_search, get_best_match, get_score
from pytickersymbols import PyTickerSymbols
import requests
from bs4 import BeautifulSoup
from pysymbolscanner.const import most_common_endings


class SymbolScanner:
    MIN_WORD_OCCURRENCE = 5
    MAX_PROCESSES = 8
    PICKLE_FILE = 'index_data.pickle'

    def __init__(self, cache):
        logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
        self.log = logging.getLogger('symbol scanner')
        self.data = None
        self.cache = cache

    def start(self):
        if self.cache and os.path.isfile(SymbolScanner.PICKLE_FILE):
            with open(SymbolScanner.PICKLE_FILE, 'rb') as handle:
                self.data = pickle.load(handle)
        else:
            self.data = self.start_index()
            self.data = self.start_metadata()
            self.data = self.start_sync()
            with open(SymbolScanner.PICKLE_FILE, 'wb') as handle:
                pickle.dump(
                    self.data, handle, protocol=pickle.HIGHEST_PROTOCOL
                )

    def start_sync(self):
        stocks = PyTickerSymbols()
        if not self.data:
            return

        stocks_py = stocks.get_all_stocks()

        names_py = list(set([stock['name'] for stock in stocks_py]))

        names_wiki = list(
            set(
                [
                    stock.wiki_name
                    for stock_list in map(
                        lambda index: self.data[index],
                        Indices.symbol_source_dict,
                    )
                    for stock in stock_list
                ]
            )
        )

        names = names_py + names_wiki
        endings = list(
            map(lambda x: ' ' + x, self.get_most_common_endings(names))
        )
        occurrences = list(self.get_most_common_occurrences(names))
        with multiprocessing.Pool(processes=self.MAX_PROCESSES) as pool:
            items = list(
                map(
                    lambda index: (index, endings, occurrences, stocks_py),
                    Indices.symbol_source_dict,
                )
            )
            sync_results = pool.map(self.worker_sync, items)
            result = dict(map(lambda val: list(val.items())[0], sync_results))
            return result

    @staticmethod
    def find_by_symbol(stock, py_stocks):
        for idx, py_stock in enumerate(py_stocks):
            for py_symbol in py_stock['symbols']:
                for symbol in stock['symbols']:
                    if symbol['yahoo'] == py_symbol['yahoo']:
                        return idx
        return -1

    @staticmethod
    def find_by_symbol_and_country(stock, py_stocks):
        for idx, py_stock in enumerate(py_stocks):
            if (
                stock['symbol'] == py_stock['symbol']
                and stock['country'] == py_stock['country']
                and get_score(stock['wiki_name'],  py_stock['name']) == 1.0
            ):
                return idx
        return -1

    def worker_sync(self, args):
        index, endings, occurrences, stocks = args
        wiki_stocks = self.data[index].copy()
        for idx, wiki_stock in enumerate(wiki_stocks):
            stock = wiki_stock.to_pyticker_symbol()
            names = [stock['name'] for stock in stocks]
            name_id = self.find_by_symbol(stock, stocks)
            if name_id != -1:
                wiki_stocks[idx].name = names[name_id]
                continue

            name_id = self.find_by_symbol_and_country(stock, stocks)
            if name_id != -1:
                wiki_stocks[idx].name = names[name_id]
                continue

            wiki_stock_name = stock['wiki_name']
            name_id, max_score = get_best_match(
                wiki_stock_name,
                names,
                word_filter=most_common_endings,
            )
            if max_score > 0.8:
                wiki_stocks[idx].name = names[name_id]
                continue

            name_id, max_score = deep_search(
                wiki_stock_name, names, occurrences
            )

            if max_score > 0.9:
                wiki_stocks[idx].name = names[name_id]
            else:
                self.log.warn(
                    f'Did not find any match for {index} - {wiki_stock_name}'
                )
                wiki_stocks[idx].name = wiki_stocks[idx].wiki_name
        return {index: wiki_stocks}

    def get_most_common_endings(self, names):
        most_words = {}
        for name in names:
            if not name.split():
                continue
            word = name.split()[-1]
            most_words[word] = most_words.get(word, 0) + 1

        items_list = [word for word in most_words.items()]
        items_list.sort(key=lambda tup: tup[1])
        return map(
            lambda x: x[0],
            filter(lambda f: f[1] >= self.MIN_WORD_OCCURRENCE, items_list),
        )

    def get_most_common_occurrences(self, names):
        most_words = {}
        for name in names:
            if not name.split():
                continue
            words = name.split()
            for word in words:
                most_words[word] = most_words.get(word, 0) + 1

        items_list = [word for word in most_words.items()]
        items_list.sort(key=lambda tup: tup[1])
        return map(
            lambda x: x[0],
            filter(lambda f: f[1] >= self.MIN_WORD_OCCURRENCE, items_list),
        )

    def start_metadata(self):
        result = {}
        with multiprocessing.Pool(processes=self.MAX_PROCESSES * 4) as pool:
            for index in self.data:
                args = map(
                    lambda stock, idx=index: (
                        stock,
                        Indices.symbol_source_dict[idx].PAGE_LANGS,
                    ),
                    self.data[index],
                )
                stocks_with_metadata = pool.map(self.worker_metadata, args)
                result[index] = stocks_with_metadata
        return result

    def worker_metadata(self, args):
        stock, langs = args
        infobox = get_merged_infobox(
            stock.wiki_name, stock.link, stock.link_lang, langs
        )
        if not infobox:
            self.log.warn(
                f'Did not find any wikipedia data for {stock.wiki_name}'
            )
            return stock
        return infobox.to_stock(stock.indices, stock.symbol)

    def start_index(self):
        with multiprocessing.Pool(processes=self.MAX_PROCESSES) as pool:
            items = list(
                map(lambda x: (x[0], x[1]), Indices.symbol_source_dict.items())
            )
            index_results = pool.map(self.worker_index, items)
            result = dict(map(lambda val: list(val.items())[0], index_results))
            return result

    def worker_index(self, args):
        key, index_source = args
        self.log.info(f'________________ parse {index_source.TITLE}')
        wiki_url = get_wiki_url(index_source.LANG, index_source.TITLE)
        response = requests.get(wiki_url)
        dfs = pd.read_html(response.text)
        table_id = self.get_table_id(dfs, index_source.ITEMS)
        if table_id == -1:
            raise RuntimeError(f'Could not find index table for {key}')
        df = dfs[table_id]
        try:
            links = self.get_wiki_links(
                response.text, table_id, index_source.COL_NAME_COMPANY
            )
        except TypeError:
            raise RuntimeError(f'Could not find company links {key} table')
        df['link'] = links
        if index_source.COL_NAME_SYMBOL is not None:
            df = df.rename(
                columns={
                    index_source.COL_NAME_COMPANY: 'name',
                    index_source.COL_NAME_SYMBOL: 'symbol',
                }
            )[['name', 'symbol', 'link']]
        else:
            df = df.rename(columns={index_source.COL_NAME_COMPANY: 'name'})
            df['symbol'] = None
            df = df[['name', 'symbol', 'link']]
        result = {key: []}
        for wiki_name, symbol, link in zip(df.name, df.symbol, df.link):
            stock = Stock.from_wiki(
                [key], wiki_name, link, index_source.LANG, symbol
            )
            result[key].append(stock)
        return result

    @staticmethod
    def get_table_id(dfs, columns):
        table_id = -1
        for idx, df in enumerate(dfs):
            if all(col in df.columns for col in columns):
                if table_id != -1:
                    raise RuntimeError(
                        'Could not detect table because of multiple matches'
                    )
                table_id = idx
        return table_id

    @staticmethod
    def get_wiki_links(html, table_id, col_name):
        soup = BeautifulSoup(html, 'html.parser')
        table = soup.findAll('table')[table_id]
        links = []
        index = -1
        for row in table.findAll('tr'):
            column_names = row.findAll('th')
            if index == -1:
                index = SymbolScanner._get_col_id(col_name, column_names)
                continue
            tr = row.findAll('td')[index]
            try:
                link = tr.find('a')['href']
                if 'action=edit' not in link:
                    links.append(link)
                else:
                    links.append('')
            except (KeyError, TypeError):
                links.append('')
        return links

    @staticmethod
    def _get_col_id(col_name, column_names):
        index = next(
            iter(
                [
                    i
                    for i, s in enumerate(column_names)
                    if col_name in s.getText()
                ]
            ),
            None,
        )
        return index



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pysymbolscanner
  Copyright 2020 Slash Gordon
  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""
import logging

import argparse
from pytickersymbols import PyTickerSymbols
from pysymbolscanner.scanner import SymbolScanner
import os
import yaml
from collections import OrderedDict

logging.getLogger('requests').setLevel(logging.WARNING)
logging.getLogger('urllib3').setLevel(logging.WARNING)


yaml.add_representer(
    OrderedDict,
    lambda self, data: self.represent_mapping(
        'tag:yaml.org,2002:map', data.items()
    ),
)


def is_valid_file(parser, arg):
    """
    Check if file exists
    :param parser: parser instance
    :param arg: path to file
    :return: return the path if exists otherwise None
    """
    if not os.path.exists(arg):
        parser.error("The file %s does not exist!" % arg)
        return None
    return arg


def get_missing_objects(index, scanner, stock_data):
    missing_stocks = list(
        filter(
            lambda x: x['name']
            not in map(
                lambda x: x['name'], stock_data.get_stocks_by_index(index)
            ),
            map(lambda y: y.to_pyticker_symbol(), scanner.data[index]),
        )
    )
    return missing_stocks


def get_wrong_objects(index, scanner, stock_data):
    wrong_stocks = list(
        filter(
            lambda x: x['name']
            not in map(lambda y: y.name, scanner.data[index]),
            map(lambda y: y, stock_data.get_stocks_by_index(index)),
        )
    )
    return wrong_stocks


def fix_missing(missings, index, stocks_yaml):
    for missing in missings:
        not_exists = True
        for stock in stocks_yaml['companies']:
            if (
                stock['name'] == missing['name']
                and index not in stock['indices']
            ):
                stock['indices'].append(index)
                stock['isins'] = missing['isins']
                stock['metadata'] = missing['metadata']
                for symbol in missing['symbols']:
                    if symbol not in stock['symbols']:
                        stock['symbols'].append(symbol)
                not_exists = False
                break
        if not_exists:
            stocks_yaml['companies'].append(missing)
    return stocks_yaml


def fix_wrong(wrongs, index, stocks_yaml):
    for wrong in wrongs:
        for stock in stocks_yaml['companies']:
            if stock['name'] == wrong['name'] and index in stock['indices']:
                stock['indices'].remove(index)
    return stocks_yaml


def symbolscanner_app():
    """
    Main entry point for symbolscanner application
    :return:
    """
    parser = argparse.ArgumentParser(description='SymbolScanner CLI')
    parser.add_argument('--cache', action='store_true', default=False)
    parser.add_argument(
        '-i',
        '--input',
        dest='input',
        action='store',
        help='Path to the input file.',
        type=lambda x: is_valid_file(parser, x),
    )
    parser.add_argument(
        '-o',
        '--output',
        dest='output',
        action='store',
        help='Path to the output file.',
    )
    args = parser.parse_args()
    stock_data = PyTickerSymbols()
    scanner = SymbolScanner(args.cache)
    scanner.start()
    indices = stock_data.get_all_indices()
    stocks_yaml = None
    with open(args.input, 'r') as in_file:
        stocks_yaml = yaml.safe_load(in_file)
        for index in indices:
            missing = get_missing_objects(index, scanner, stock_data)
            wrong = get_wrong_objects(index, scanner, stock_data)
            stocks_yaml = fix_wrong(wrong, index, stocks_yaml)
            stocks_yaml = fix_missing(missing, index, stocks_yaml)
    with open(args.output, 'w', encoding='latin1') as out_file:
        yaml.dump(stocks_yaml, out_file, sort_keys=False)



from difflib import SequenceMatcher


def _remove(word, word_filter):
    for old in word_filter:
        if word.endswith(old):
            word = word.replace(old, '')
    return word


def _search(sentence_a, sentence_b, ignore):
    words_a = [sentence_a] if not sentence_a.split() else sentence_a.split()
    words_b = [sentence_b] if not sentence_b.split() else sentence_b.split()
    words_a = list(filter(lambda x: x not in ignore, words_a))
    words_b = list(filter(lambda x: x not in ignore, words_b))
    for word_a in words_a:
        for word_b in words_b:
            if word_a == word_b:
                return True
    return False


def get_best_match(word, items, word_filter=[]):
    if word_filter:
        word = _remove(word, word_filter)
        items = list(map(lambda x: _remove(x, word_filter), items))

    socres = list(get_scores(word, items))
    max_score = max(socres)
    idx_max_score = socres.index(max_score)
    return idx_max_score, max_score


def deep_search(word, items, ignore=[]):
    result = []
    for idx, item in enumerate(items):
        if _search(item.lower(), word.lower(), ignore):
            result.append(idx)
    if not result:
        return -1, -1
    idx_max_score, max_score = get_best_match(
        word, map(lambda idx: items[idx], result)
    )
    return result[idx_max_score], max_score


def get_scores(word, items):
    return map(
        lambda item: get_score(word, item),
        items,
    )


def get_score(word_a, word_b):
    return SequenceMatcher(None, word_a.lower(), word_b.lower()).ratio()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" pysymbolscanner
  Copyright 2020 Slash Gordon
  Use of this source code is governed by an MIT-style license that
  can be found in the LICENSE file.
"""
from dataclasses import dataclass
from pytickersymbols import Statics


@dataclass
class IndexSource:
    TITLE: str
    ITEMS: []
    LANG: str
    COL_NAME_SYMBOL: str
    COL_NAME_COMPANY: str
    PAGE_LANGS: []


class Indices:

    symbol_source_dict = {
        Statics.Indices.DE_SDAX: IndexSource(
            'SDAX',
            ['Logo', 'Name', 'Branche'],
            'de',
            None,
            'Name',
            ['de', 'en'],
        ),
        Statics.Indices.RU_MOEX: IndexSource(
            'RTS_Index',
            ['Logo', 'Unternehmen', 'Branche'],
            'de',
            None,
            'Unternehmen',
            ['de', 'en'],
        ),
        Statics.Indices.GB_FTSE: IndexSource(
            'FTSE_100_Index',
            ['Company', 'EPIC'],
            'en',
            'EPIC',
            'Company',
            ['en', 'de'],
        ),
        Statics.Indices.FI_OMX_25: IndexSource(
            'OMX_Helsinki_25',
            ['Company', 'Ticker symbol'],
            'en',
            'Ticker symbol',
            'Company',
            ['en', 'de'],
        ),
        Statics.Indices.EU_50: IndexSource(
            'EURO_STOXX_50',
            ['Name', 'Main listing', 'Ticker', 'Industry'],
            'en',
            'Ticker',
            'Name',
            ['en', 'de', 'fr'],
        ),
        Statics.Indices.US_SP_100: IndexSource(
            'S%26P_100',
            ['Name', 'Symbol'],
            'en',
            'Symbol',
            'Name',
            ['en'],
        ),
        Statics.Indices.ES_IBEX_35: IndexSource(
            'IBEX_35',
            ['Company', 'Ticker', 'Sector'],
            'en',
            'Ticker',
            'Company',
            ['en', 'es'],
        ),
        Statics.Indices.US_DOW: IndexSource(
            'Dow_Jones_Industrial_Average',
            ['Symbol', 'Company', 'Industry'],
            'en',
            'Symbol',
            'Company',
            ['en'],
        ),
        Statics.Indices.DE_DAX: IndexSource(
            'DAX',
            ['Logo', 'Name', 'Branche', 'Symbol'],
            'de',
            'Symbol',
            'Name',
            ['de', 'en'],
        ),
        Statics.Indices.FR_CAC_60: IndexSource(
            'CAC_Mid_60',
            ['Unternehmen', 'Branche', 'Logo'],
            'de',
            None,
            'Unternehmen',
            ['de', 'en', 'fr'],
        ),
        Statics.Indices.DE_TECDAX: IndexSource(
            'TecDAX',
            ['Logo', 'Name', 'Branche'],
            'de',
            'Symbol[11]',
            'Name',
            ['de', 'en'],
        ),
        Statics.Indices.US_NASDAQ: IndexSource(
            'NASDAQ-100',
            ['Company', 'Ticker'],
            'en',
            'Ticker',
            'Company',
            ['en'],
        ),
        Statics.Indices.CH_20: IndexSource(
            'Swiss_Market_Index',
            ['Name', 'Industry', 'Ticker'],
            'en',
            'Ticker',
            'Name',
            ['en', 'de', 'fr'],
        ),
        Statics.Indices.FR_CAC_40: IndexSource(
            'CAC_40',
            ['Company', 'Sector', 'Ticker'],
            'en',
            'Ticker',
            'Company',
            ['en', 'fr', 'de'],
        ),
        Statics.Indices.US_SP_500: IndexSource(
            'List_of_S%26P_500_companies',
            ['Symbol', 'Security', 'Founded', 'Headquarters Location'],
            'en',
            'Symbol',
            'Security',
            ['en'],
        ),
        Statics.Indices.SE_OMX_30: IndexSource(
            'OMX_Stockholm_30',
            ['Logo', 'Name', 'Branche'],
            'de',
            None,
            'Name',
            ['de', 'en'],
        ),
        Statics.Indices.BE_20: IndexSource(
            'BEL_20',
            ['Logo', 'Name', 'Branche'],
            'de',
            None,
            'Name',
            ['de', 'en', 'fr'],
        ),
        Statics.Indices.DE_MDAX: IndexSource(
            'MDAX',
            ['Logo', 'Name', 'Branche'],
            'de',
            None,
            'Name',
            ['de', 'en'],
        ),
        Statics.Indices.NL_AEX: IndexSource(
            'AEX_index',
            ['Company', 'Ticker symbol'],
            'en',
            'Ticker symbol',
            'Company',
            ['en', 'de'],
        ),
    }



most_common_endings = [
        'GmbH & Co. Kommanditgesellschaft auf Aktien',
        'GmbH & Co. KGaA',
        'AG & Co. KGaA',
        'Aktiengeselschaft',
        'SA',
        'S.A.',
        'AG',
        'Ag',
        'ag',
        'SE',
        'S.E.',
        'plc',
        'Plc',
        'PLC',
        'GmbH',
        'KGaA',
        'Corp.',
        'Co.',
        'Inc.',
        '(Class C)',
        '(Class B)',
        '(Class A)',
        '(A)',
        '(B)',
        '(C)',
        'OJSC',
        'PJSC',
        'S.p.A.',
        'Oyj',
        '(Unternehmen)',
        '(Company)',
    ]

blocklist_search = ['SDAX', 'MDAX', 'TecDAX', 'S&P 500']


def remove_most_common_endings(company):
    for end in most_common_endings:
        company = company.replace(f' {end}', '')
    return company.replace(',', '')
